@@ -14,7 +14,8 @@ from telegram import (
 from typing import Dict, Text, Any, List, Optional, Callable, Awaitable
 
 from rasa.core.channels.channel import InputChannel, UserMessage, OutputChannel
-from rasa.core.constants import INTENT_MESSAGE_PREFIX, USER_INTENT_RESTART
+from rasa.shared.constants import INTENT_MESSAGE_PREFIX
+from rasa.shared.core.constants import USER_INTENT_RESTART
 
 logger = logging.getLogger(__name__)
 
@@ -22,6 +23,7 @@ logger = logging.getLogger(__name__)
 class TelegramOutput(Bot, OutputChannel):
     """Output channel for Telegram"""
 
+    # skipcq: PYL-W0236
     @classmethod
     def name(cls) -> Text:
         return "telegram"
