@@ -1,17 +1,6 @@
 from typing import Optional, Text
 
-from rasa.exceptions import RasaException
-
-
-class RasaCoreException(RasaException):
-    """Basic exception for errors raised by Rasa Core."""
-
-
-class StoryParseError(RasaCoreException, ValueError):
-    """Raised if there is an error while parsing a story file."""
-
-    def __init__(self, message) -> None:
-        self.message = message
+from rasa.shared.exceptions import RasaCoreException
 
 
 class UnsupportedDialogueModelError(RasaCoreException):
@@ -24,6 +13,7 @@ class UnsupportedDialogueModelError(RasaCoreException):
     def __init__(self, message: Text, model_version: Optional[Text] = None) -> None:
         self.message = message
         self.model_version = model_version
+        super(UnsupportedDialogueModelError, self).__init__()
 
     def __str__(self) -> Text:
         return self.message
@@ -38,3 +28,4 @@ class AgentNotReady(RasaCoreException):
 
     def __init__(self, message: Text) -> None:
         self.message = message
+        super(AgentNotReady, self).__init__()
