@@ -6,8 +6,8 @@ import os
 from async_generator import asynccontextmanager
 from typing import Text, Union, Optional, AsyncGenerator
 
+import rasa.shared.utils.common
 from rasa.core.constants import DEFAULT_LOCK_LIFETIME
-from rasa.utils import common
 from rasa.core.lock import TicketLock
 from rasa.utils.endpoints import EndpointConfig
 
@@ -26,8 +26,8 @@ DEFAULT_SOCKET_TIMEOUT_IN_SECONDS = 10
 class LockError(Exception):
     """Exception that is raised when a lock cannot be acquired.
 
-     Attributes:
-          message (str): explanation of which `conversation_id` raised the error
+    Attributes:
+         message (str): explanation of which `conversation_id` raised the error
     """
 
     pass
@@ -274,18 +274,22 @@ def _create_from_endpoint_config(
     elif endpoint_config.type == "redis":
         lock_store = RedisLockStore(host=endpoint_config.url, **endpoint_config.kwargs)
     else:
-        lock_store = _load_from_module_string(endpoint_config)
+        lock_store = _load_from_module_name_in_endpoint_config(endpoint_config)
 
     logger.debug(f"Connected to lock store '{lock_store.__class__.__name__}'.")
 
     return lock_store
 
 
-def _load_from_module_string(endpoint_config: EndpointConfig) -> "LockStore":
-    """Given the name of a `LockStore` module tries to retrieve it."""
+def _load_from_module_name_in_endpoint_config(
+    endpoint_config: EndpointConfig,
+) -> "LockStore":
+    """Retrieve a `LockStore` based on its class name."""
 
     try:
-        lock_store_class = common.class_from_module_path(endpoint_config.type)
+        lock_store_class = rasa.shared.utils.common.class_from_module_path(
+            endpoint_config.type
+        )
         return lock_store_class(endpoint_config=endpoint_config)
     except (AttributeError, ImportError) as e:
         raise Exception(
