@@ -8,33 +8,37 @@ from functools import partial
 from multiprocessing import Process
 from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union, Set
 
-import numpy as np
-from aiohttp import ClientError
-from colorclass import Color
 
-from rasa.nlu.training_data.loading import MARKDOWN, RASA
 from sanic import Sanic, response
 from sanic.exceptions import NotFound
 from terminaltables import AsciiTable, SingleTable
-
+import numpy as np
+from aiohttp import ClientError
+from colorclass import Color
 import questionary
-import rasa.cli.utils
 from questionary import Choice, Form, Question
 
-from rasa.cli import utils as cli_utils
-from rasa.core import constants, run, train, utils
-from rasa.core.actions.action import ACTION_LISTEN_NAME, default_action_names
-from rasa.core.channels.channel import UserMessage
-from rasa.core.constants import (
-    DEFAULT_SERVER_FORMAT,
-    DEFAULT_SERVER_PORT,
-    DEFAULT_SERVER_URL,
+from rasa import telemetry
+import rasa.shared.utils.cli
+import rasa.shared.utils.io
+import rasa.cli.utils
+import rasa.shared.data
+from rasa.shared.nlu.constants import TEXT, INTENT_NAME_KEY
+from rasa.shared.nlu.training_data.loading import MARKDOWN, RASA, RASA_YAML
+from rasa.shared.core.constants import (
+    USER_INTENT_RESTART,
+    ACTION_LISTEN_NAME,
+    LOOP_NAME,
+    ACTIVE_LOOP,
+    LOOP_REJECTED,
     REQUESTED_SLOT,
-    UTTER_PREFIX,
+    LOOP_INTERRUPTED,
 )
-from rasa.core.domain import Domain
-import rasa.core.events
-from rasa.core.events import (
+from rasa.core import run, train, utils
+from rasa.core.constants import DEFAULT_SERVER_FORMAT, DEFAULT_SERVER_PORT
+from rasa.shared.core.domain import Domain
+import rasa.shared.core.events
+from rasa.shared.core.events import (
     ActionExecuted,
     ActionReverted,
     BotUttered,
@@ -43,21 +47,22 @@ from rasa.core.events import (
     UserUttered,
     UserUtteranceReverted,
 )
-from rasa.core.interpreter import INTENT_MESSAGE_PREFIX, NaturalLanguageInterpreter
-from rasa.core.trackers import EventVerbosity, DialogueStateTracker
-from rasa.core.training import visualization
-from rasa.core.training.visualization import (
+import rasa.core.interpreter
+from rasa.shared.constants import INTENT_MESSAGE_PREFIX, DEFAULT_SENDER_ID, UTTER_PREFIX
+from rasa.shared.core.trackers import EventVerbosity, DialogueStateTracker
+from rasa.shared.core.training_data import visualization
+from rasa.shared.core.training_data.visualization import (
     VISUALIZATION_TEMPLATE_PATH,
     visualize_neighborhood,
 )
 from rasa.core.utils import AvailableEndpoints
-from rasa.importers.rasa import TrainingDataImporter
+from rasa.shared.importers.rasa import TrainingDataImporter
 from rasa.utils.common import update_sanic_log_level
 from rasa.utils.endpoints import EndpointConfig
 
 # noinspection PyProtectedMember
-from rasa.nlu.training_data import loading
-from rasa.nlu.training_data.message import Message
+from rasa.shared.nlu.training_data import loading
+from rasa.shared.nlu.training_data.message import Message
 
 # WARNING: This command line UI is using an external library
 # communicating with the shell - these functions are hard to test
@@ -71,9 +76,9 @@ logger = logging.getLogger(__name__)
 MAX_VISUAL_HISTORY = 3
 
 PATHS = {
-    "stories": "data/stories.md",
-    "nlu": "data/nlu.md",
-    "backup": "data/nlu_interactive.md",
+    "stories": "data/stories.yml",
+    "nlu": "data/nlu.yml",
+    "backup": "data/nlu_interactive.yml",
     "domain": "domain.yml",
 }
 
@@ -200,9 +205,9 @@ async def send_action(
             if action_name in NEW_TEMPLATES:
                 warning_questions = questionary.confirm(
                     f"WARNING: You have created a new action: '{action_name}', "
-                    f"with matching template: '{[*NEW_TEMPLATES[action_name]][0]}'. "
+                    f"with matching response: '{[*NEW_TEMPLATES[action_name]][0]}'. "
                     f"This action will not return its message in this session, "
-                    f"but the new utterance will be saved to your domain file "
+                    f"but the new response will be saved to your domain file "
                     f"when you exit and save this session. "
                     f"You do not need to do anything further."
                 )
@@ -257,7 +262,7 @@ def format_bot_output(message: BotUttered) -> Text:
 
     if data.get("buttons"):
         output += "\nButtons:"
-        choices = cli_utils.button_choices_from_message_data(
+        choices = rasa.cli.utils.button_choices_from_message_data(
             data, allow_free_text_input=True
         )
         for choice in choices:
@@ -266,13 +271,13 @@ def format_bot_output(message: BotUttered) -> Text:
     if data.get("elements"):
         output += "\nElements:"
         for idx, element in enumerate(data.get("elements")):
-            element_str = cli_utils.element_to_string(element, idx)
+            element_str = rasa.cli.utils.element_to_string(element, idx)
             output += "\n" + element_str
 
     if data.get("quick_replies"):
         output += "\nQuick replies:"
         for idx, element in enumerate(data.get("quick_replies")):
-            element_str = cli_utils.element_to_string(element, idx)
+            element_str = rasa.cli.utils.element_to_string(element, idx)
             output += "\n" + element_str
     return output
 
@@ -320,14 +325,21 @@ async def _ask_questions(
 def _selection_choices_from_intent_prediction(
     predictions: List[Dict[Text, Any]]
 ) -> List[Dict[Text, Any]]:
-    """"Given a list of ML predictions create a UI choice list."""
+    """Given a list of ML predictions create a UI choice list."""
 
-    sorted_intents = sorted(predictions, key=lambda k: (-k["confidence"], k["name"]))
+    sorted_intents = sorted(
+        predictions, key=lambda k: (-k["confidence"], k[INTENT_NAME_KEY])
+    )
 
     choices = []
     for p in sorted_intents:
-        name_with_confidence = f'{p.get("confidence"):03.2f} {p.get("name"):40}'
-        choice = {"name": name_with_confidence, "value": p.get("name")}
+        name_with_confidence = (
+            f'{p.get("confidence"):03.2f} {p.get(INTENT_NAME_KEY):40}'
+        )
+        choice = {
+            INTENT_NAME_KEY: name_with_confidence,
+            "value": p.get(INTENT_NAME_KEY),
+        }
         choices.append(choice)
 
     return choices
@@ -358,10 +370,8 @@ async def _request_free_text_utterance(
 ) -> Text:
 
     question = questionary.text(
-        message=(
-            f"Please type the message for your new utterance template '{action}':"
-        ),
-        validate=io_utils.not_empty_validator("Please enter a template message"),
+        message=(f"Please type the message for your new bot response '{action}':"),
+        validate=io_utils.not_empty_validator("Please enter a response"),
     )
     return await _ask_questions(question, conversation_id, endpoint)
 
@@ -418,15 +428,15 @@ async def _request_intent_from_user(
 
     predictions = latest_message.get("parse_data", {}).get("intent_ranking", [])
 
-    predicted_intents = {p["name"] for p in predictions}
+    predicted_intents = {p[INTENT_NAME_KEY] for p in predictions}
 
     for i in intents:
         if i not in predicted_intents:
-            predictions.append({"name": i, "confidence": 0.0})
+            predictions.append({INTENT_NAME_KEY: i, "confidence": 0.0})
 
     # convert intents to ui list and add <other> as a free text alternative
     choices = [
-        {"name": "<create_new_intent>", "value": OTHER_INTENT}
+        {INTENT_NAME_KEY: "<create_new_intent>", "value": OTHER_INTENT}
     ] + _selection_choices_from_intent_prediction(predictions)
 
     intent_name = await _request_selection_from_intents(
@@ -435,11 +445,12 @@ async def _request_intent_from_user(
 
     if intent_name == OTHER_INTENT:
         intent_name = await _request_free_text_intent(conversation_id, endpoint)
-        selected_intent = {"name": intent_name, "confidence": 1.0}
+        selected_intent = {INTENT_NAME_KEY: intent_name, "confidence": 1.0}
     else:
         # returns the selected intent with the original probability value
         selected_intent = next(
-            (x for x in predictions if x["name"] == intent_name), {"name": None}
+            (x for x in predictions if x[INTENT_NAME_KEY] == intent_name),
+            {INTENT_NAME_KEY: None},
         )
 
     return selected_intent
@@ -481,7 +492,7 @@ def _chat_history_table(events: List[Dict[Text, Any]]) -> Text:
 
     def format_user_msg(user_event: UserUttered, max_width: int) -> Text:
         intent = user_event.intent or {}
-        intent_name = intent.get("name", "")
+        intent_name = intent.get(INTENT_NAME_KEY, "")
         _confidence = intent.get("confidence", 1.0)
         _md = _as_md_message(user_event.parse_data)
 
@@ -562,13 +573,28 @@ def _slot_history(tracker_dump: Dict[Text, Any]) -> List[Text]:
 
     slot_strings = []
     for k, s in tracker_dump.get("slots", {}).items():
-        colored_value = cli_utils.wrap_with_color(
-            str(s), color=rasa.cli.utils.bcolors.WARNING
+        colored_value = rasa.shared.utils.io.wrap_with_color(
+            str(s), color=rasa.shared.utils.io.bcolors.WARNING
         )
         slot_strings.append(f"{k}: {colored_value}")
     return slot_strings
 
 
+def _retry_on_error(
+    func: Callable, export_path: Text, *args: Any, **kwargs: Any
+) -> None:
+    while True:
+        try:
+            return func(export_path, *args, **kwargs)
+        except OSError as e:
+            answer = questionary.confirm(
+                f"Failed to export '{export_path}': {e}. Please make sure 'rasa' "
+                f"has read and write access to this file. Would you like to retry?"
+            ).ask()
+            if not answer:
+                raise e
+
+
 async def _write_data_to_file(conversation_id: Text, endpoint: EndpointConfig):
     """Write stories and nlu data to file."""
 
@@ -580,9 +606,9 @@ async def _write_data_to_file(conversation_id: Text, endpoint: EndpointConfig):
     serialised_domain = await retrieve_domain(endpoint)
     domain = Domain.from_dict(serialised_domain)
 
-    await _write_stories_to_file(story_path, events, domain)
-    await _write_nlu_to_file(nlu_path, events)
-    await _write_domain_to_file(domain_path, events, domain)
+    _retry_on_error(_write_stories_to_file, story_path, events, domain)
+    _retry_on_error(_write_nlu_to_file, nlu_path, events)
+    _retry_on_error(_write_domain_to_file, domain_path, events, domain)
 
     logger.info("Successfully wrote stories and NLU data")
 
@@ -672,6 +698,8 @@ async def _request_action_from_user(
 
 
 def _request_export_info() -> Tuple[Text, Text, Text]:
+    import rasa.shared.data
+
     """Request file path and export stories & nlu data to that path"""
 
     # export training data and quit
@@ -681,8 +709,10 @@ def _request_export_info() -> Tuple[Text, Text, Text]:
             "will append the stories)",
             default=PATHS["stories"],
             validate=io_utils.file_type_validator(
-                [".md"],
-                "Please provide a valid export path for the stories, e.g. 'stories.md'.",
+                rasa.shared.data.MARKDOWN_FILE_EXTENSIONS
+                + rasa.shared.data.YAML_FILE_EXTENSIONS,
+                "Please provide a valid export path for the stories, "
+                "e.g. 'stories.yml'.",
             ),
         ),
         export_nlu=questionary.text(
@@ -690,8 +720,9 @@ def _request_export_info() -> Tuple[Text, Text, Text]:
             "merge learned data with previous training examples)",
             default=PATHS["nlu"],
             validate=io_utils.file_type_validator(
-                [".md", ".json"],
-                "Please provide a valid export path for the NLU data, e.g. 'nlu.md'.",
+                list(rasa.shared.data.TRAINING_DATA_EXTENSIONS),
+                "Please provide a valid export path for the NLU data, "
+                "e.g. 'nlu.yml'.",
             ),
         ),
         export_domain=questionary.text(
@@ -699,8 +730,9 @@ def _request_export_info() -> Tuple[Text, Text, Text]:
             "will be overwritten)",
             default=PATHS["domain"],
             validate=io_utils.file_type_validator(
-                [".yml", ".yaml"],
-                "Please provide a valid export path for the domain file, e.g. 'domain.yml'.",
+                rasa.shared.data.YAML_FILE_EXTENSIONS,
+                "Please provide a valid export path for the domain file, "
+                "e.g. 'domain.yml'.",
             ),
         ),
     )
@@ -739,7 +771,7 @@ def _collect_messages(events: List[Dict[Text, Any]]) -> List[Message]:
     """Collect the message text and parsed data from the UserMessage events
     into a list"""
 
-    import rasa.nlu.training_data.util as rasa_nlu_training_data_utils
+    import rasa.shared.nlu.training_data.util as rasa_nlu_training_data_utils
 
     messages = []
 
@@ -747,7 +779,9 @@ def _collect_messages(events: List[Dict[Text, Any]]) -> List[Message]:
         if event.get("event") == UserUttered.type_name:
             data = event.get("parse_data", {})
             rasa_nlu_training_data_utils.remove_untrainable_entities_from(data)
-            msg = Message.build(data["text"], data["intent"]["name"], data["entities"])
+            msg = Message.build(
+                data["text"], data["intent"][INTENT_NAME_KEY], data["entities"]
+            )
             messages.append(msg)
         elif event.get("event") == UserUtteranceReverted.type_name and messages:
             messages.pop()  # user corrected the nlu, remove incorrect example
@@ -761,24 +795,42 @@ def _collect_actions(events: List[Dict[Text, Any]]) -> List[Dict[Text, Any]]:
     return [evt for evt in events if evt.get("event") == ActionExecuted.type_name]
 
 
-async def _write_stories_to_file(
+def _write_stories_to_file(
     export_story_path: Text, events: List[Dict[Text, Any]], domain: Domain
 ) -> None:
     """Write the conversation of the conversation_id to the file paths."""
+    from rasa.shared.core.training_data.story_reader.yaml_story_reader import (
+        YAMLStoryReader,
+    )
+    from rasa.shared.core.training_data.story_writer.yaml_story_writer import (
+        YAMLStoryWriter,
+    )
+    from rasa.shared.core.training_data.story_writer.markdown_story_writer import (
+        MarkdownStoryWriter,
+    )
 
     sub_conversations = _split_conversation_at_restarts(events)
 
     io_utils.create_path(export_story_path)
 
+    if rasa.shared.data.is_likely_yaml_file(export_story_path):
+        writer = YAMLStoryWriter()
+    else:
+        writer = MarkdownStoryWriter()
+
+    should_append_stories = False
     if os.path.exists(export_story_path):
         append_write = "a"  # append if already exists
+        should_append_stories = True
     else:
         append_write = "w"  # make a new file if not
 
-    with open(export_story_path, append_write, encoding=io_utils.DEFAULT_ENCODING) as f:
+    with open(
+        export_story_path, append_write, encoding=rasa.shared.utils.io.DEFAULT_ENCODING
+    ) as f:
         i = 1
         for conversation in sub_conversations:
-            parsed_events = rasa.core.events.deserialise_events(conversation)
+            parsed_events = rasa.shared.core.events.deserialise_events(conversation)
             tracker = DialogueStateTracker.from_events(
                 f"interactive_story_{i}", evts=parsed_events, slots=domain.slots
             )
@@ -787,7 +839,14 @@ async def _write_stories_to_file(
                 isinstance(event, UserUttered) for event in tracker.applied_events()
             ):
                 i += 1
-                f.write("\n" + tracker.export_stories(SAVE_IN_E2E))
+                f.write(
+                    "\n"
+                    + tracker.export_stories(
+                        writer=writer,
+                        should_append_stories=should_append_stories,
+                        e2e=SAVE_IN_E2E,
+                    )
+                )
 
 
 def _filter_messages(msgs: List[Message]) -> List[Message]:
@@ -795,16 +854,14 @@ def _filter_messages(msgs: List[Message]) -> List[Message]:
 
     filtered_messages = []
     for msg in msgs:
-        if not msg.text.startswith(INTENT_MESSAGE_PREFIX):
+        if not msg.get(TEXT).startswith(INTENT_MESSAGE_PREFIX):
             filtered_messages.append(msg)
     return filtered_messages
 
 
-async def _write_nlu_to_file(
-    export_nlu_path: Text, events: List[Dict[Text, Any]]
-) -> None:
+def _write_nlu_to_file(export_nlu_path: Text, events: List[Dict[Text, Any]]) -> None:
     """Write the nlu data of the conversation_id to the file paths."""
-    from rasa.nlu.training_data import TrainingData
+    from rasa.shared.nlu.training_data.training_data import TrainingData
 
     msgs = _collect_messages(events)
     msgs = _filter_messages(msgs)
@@ -824,22 +881,26 @@ async def _write_nlu_to_file(
     # need to guess the format of the file before opening it to avoid a read
     # in a write
     nlu_format = _get_nlu_target_format(export_nlu_path)
-    if nlu_format == MARKDOWN:
+    if nlu_format == RASA_YAML:
+        stringified_training_data = nlu_data.nlu_as_yaml()
+    elif nlu_format == MARKDOWN:
         stringified_training_data = nlu_data.nlu_as_markdown()
     else:
         stringified_training_data = nlu_data.nlu_as_json()
 
-    io_utils.write_text_file(stringified_training_data, export_nlu_path)
+    rasa.shared.utils.io.write_text_file(stringified_training_data, export_nlu_path)
 
 
 def _get_nlu_target_format(export_path: Text) -> Text:
     guessed_format = loading.guess_format(export_path)
 
-    if guessed_format not in {MARKDOWN, RASA}:
-        if export_path.endswith(".json"):
+    if guessed_format not in {MARKDOWN, RASA, RASA_YAML}:
+        if rasa.shared.data.is_likely_json_file(export_path):
             guessed_format = RASA
-        else:
+        elif rasa.shared.data.is_likely_markdown_file(export_path):
             guessed_format = MARKDOWN
+        elif rasa.shared.data.is_likely_yaml_file(export_path):
+            guessed_format = RASA_YAML
 
     return guessed_format
 
@@ -858,7 +919,7 @@ def _intents_from_messages(messages: List[Message]) -> Set[Text]:
     return distinct_intents
 
 
-async def _write_domain_to_file(
+def _write_domain_to_file(
     domain_path: Text, events: List[Dict[Text, Any]], old_domain: Domain
 ) -> None:
     """Write an updated domain file to the file path."""
@@ -871,7 +932,12 @@ async def _write_domain_to_file(
 
     # TODO for now there is no way to distinguish between action and form
     collected_actions = list(
-        {e["name"] for e in actions if e["name"] not in default_action_names()}
+        {
+            e["name"]
+            for e in actions
+            if e["name"] not in rasa.shared.core.constants.DEFAULT_ACTION_NAMES
+            and e["name"] not in old_domain.form_names
+        }
     )
 
     new_domain = Domain(
@@ -880,7 +946,7 @@ async def _write_domain_to_file(
         slots=[],
         templates=templates,
         action_names=collected_actions,
-        form_names=[],
+        forms=[],
     )
 
     old_domain.merge(new_domain).persist_clean(domain_path)
@@ -931,21 +997,20 @@ async def _predict_till_next_listen(
         if last_event.get("event") == BotUttered.type_name and last_event["data"].get(
             "buttons", None
         ):
-            response = _get_button_choice(last_event)
-            if response != cli_utils.FREE_TEXT_INPUT_PROMPT:
-                await send_message(endpoint, conversation_id, response)
+            user_selection = _get_button_choice(last_event)
+            if user_selection != rasa.cli.utils.FREE_TEXT_INPUT_PROMPT:
+                await send_message(endpoint, conversation_id, user_selection)
 
 
 def _get_button_choice(last_event: Dict[Text, Any]) -> Text:
     data = last_event["data"]
     message = last_event.get("text", "")
 
-    choices = cli_utils.button_choices_from_message_data(
+    choices = rasa.cli.utils.button_choices_from_message_data(
         data, allow_free_text_input=True
     )
     question = questionary.select(message, choices)
-    response = cli_utils.payload_from_button_question(question)
-    return response
+    return rasa.cli.utils.payload_from_button_question(question)
 
 
 async def _correct_wrong_nlu(
@@ -989,8 +1054,8 @@ async def _correct_wrong_action(
 def _form_is_rejected(action_name: Text, tracker: Dict[Text, Any]) -> bool:
     """Check if the form got rejected with the most recent action name."""
     return (
-        tracker.get("active_form", {}).get("name")
-        and action_name != tracker["active_form"]["name"]
+        tracker.get(ACTIVE_LOOP, {}).get(LOOP_NAME)
+        and action_name != tracker[ACTIVE_LOOP][LOOP_NAME]
         and action_name != ACTION_LISTEN_NAME
     )
 
@@ -998,9 +1063,9 @@ def _form_is_rejected(action_name: Text, tracker: Dict[Text, Any]) -> bool:
 def _form_is_restored(action_name: Text, tracker: Dict[Text, Any]) -> bool:
     """Check whether the form is called again after it was rejected."""
     return (
-        tracker.get("active_form", {}).get("rejected")
+        tracker.get(ACTIVE_LOOP, {}).get(LOOP_REJECTED)
         and tracker.get("latest_action_name") == ACTION_LISTEN_NAME
-        and action_name == tracker.get("active_form", {}).get("name")
+        and action_name == tracker.get(ACTIVE_LOOP, {}).get(LOOP_NAME)
     )
 
 
@@ -1024,10 +1089,15 @@ async def _confirm_form_validation(
     if not validate_input:
         # notify form action to skip validation
         await send_event(
-            endpoint, conversation_id, {"event": "form_validation", "validate": False}
+            endpoint,
+            conversation_id,
+            {
+                "event": rasa.shared.core.events.LoopInterrupted.type_name,
+                LOOP_INTERRUPTED: True,
+            },
         )
 
-    elif not tracker.get("active_form", {}).get("validate"):
+    elif tracker.get(ACTIVE_LOOP, {}).get(LOOP_INTERRUPTED):
         # handle contradiction with learned behaviour
         warning_question = questionary.confirm(
             "ERROR: FormPolicy predicted no form validation "
@@ -1041,7 +1111,12 @@ async def _confirm_form_validation(
         await _ask_questions(warning_question, conversation_id, endpoint)
         # notify form action to validate an input
         await send_event(
-            endpoint, conversation_id, {"event": "form_validation", "validate": True}
+            endpoint,
+            conversation_id,
+            {
+                "event": rasa.shared.core.events.LoopInterrupted.type_name,
+                LOOP_INTERRUPTED: False,
+            },
         )
 
 
@@ -1079,7 +1154,7 @@ async def _validate_action(
             conversation_id,
             {
                 "event": "action_execution_rejected",
-                "name": tracker["active_form"]["name"],
+                LOOP_NAME: tracker[ACTIVE_LOOP][LOOP_NAME],
             },
         )
 
@@ -1098,7 +1173,7 @@ async def _validate_action(
 
 def _as_md_message(parse_data: Dict[Text, Any]) -> Text:
     """Display the parse data of a message in markdown format."""
-    from rasa.nlu.training_data.formats import MarkdownWriter
+    from rasa.shared.nlu.training_data.formats.readerwriter import TrainingDataWriter
 
     if parse_data.get("text", "").startswith(INTENT_MESSAGE_PREFIX):
         return parse_data["text"]
@@ -1106,7 +1181,7 @@ def _as_md_message(parse_data: Dict[Text, Any]) -> Text:
     if not parse_data.get("entities"):
         parse_data["entities"] = []
 
-    return MarkdownWriter.generate_message_md(parse_data)
+    return TrainingDataWriter.generate_message(parse_data)
 
 
 def _validate_user_regex(latest_message: Dict[Text, Any], intents: List[Text]) -> bool:
@@ -1116,7 +1191,7 @@ def _validate_user_regex(latest_message: Dict[Text, Any], intents: List[Text]) -
     `/greet`. Return `True` if the intent is a known one."""
 
     parse_data = latest_message.get("parse_data", {})
-    intent = parse_data.get("intent", {}).get("name")
+    intent = parse_data.get("intent", {}).get(INTENT_NAME_KEY)
 
     if intent in intents:
         return True
@@ -1133,7 +1208,7 @@ async def _validate_user_text(
 
     parse_data = latest_message.get("parse_data", {})
     text = _as_md_message(parse_data)
-    intent = parse_data.get("intent", {}).get("name")
+    intent = parse_data.get("intent", {}).get(INTENT_NAME_KEY)
     entities = parse_data.get("entities", [])
     if entities:
         message = (
@@ -1201,7 +1276,7 @@ async def _correct_entities(
     """Validate the entities of a user message.
 
     Returns the corrected entities"""
-    from rasa.nlu.training_data.formats import MarkdownReader
+    from rasa.shared.nlu.training_data import entities_parser
 
     parse_original = latest_message.get("parse_data", {})
     entity_str = _as_md_message(parse_original)
@@ -1210,8 +1285,7 @@ async def _correct_entities(
     )
 
     annotation = await _ask_questions(question, conversation_id, endpoint)
-    # noinspection PyProtectedMember
-    parse_annotated = MarkdownReader().parse_training_example(annotation)
+    parse_annotated = entities_parser.parse_training_example(annotation)
 
     corrected_entities = _merge_annotated_and_original_entities(
         parse_annotated, parse_original
@@ -1251,7 +1325,7 @@ async def _enter_user_message(conversation_id: Text, endpoint: EndpointConfig) -
 
     message = await _ask_questions(question, conversation_id, endpoint, lambda a: not a)
 
-    if message == (INTENT_MESSAGE_PREFIX + constants.USER_INTENT_RESTART):
+    if message == (INTENT_MESSAGE_PREFIX + USER_INTENT_RESTART):
         raise RestartConversation()
 
     await send_message(endpoint, conversation_id, message)
@@ -1310,7 +1384,7 @@ async def _fetch_events(
             events = tracker.get("events", [])
 
             for conversation in _split_conversation_at_restarts(events):
-                parsed_events = rasa.core.events.deserialise_events(conversation)
+                parsed_events = rasa.shared.core.events.deserialise_events(conversation)
                 event_sequences.append(parsed_events)
         else:
             event_sequences.append(conversation_id)
@@ -1362,7 +1436,7 @@ def _print_help(skip_visualization: bool) -> None:
     else:
         visualization_help = ""
 
-    rasa.cli.utils.print_success(
+    rasa.shared.utils.cli.print_success(
         f"Bot loaded. {visualization_help}\n"
         f"Type a message and press enter "
         f"(press 'Ctr-c' to exit)."
@@ -1372,7 +1446,7 @@ def _print_help(skip_visualization: bool) -> None:
 async def record_messages(
     endpoint: EndpointConfig,
     file_importer: TrainingDataImporter,
-    conversation_id: Text = UserMessage.DEFAULT_SENDER_ID,
+    conversation_id: Text = DEFAULT_SENDER_ID,
     max_message_limit: Optional[int] = None,
     skip_visualization: bool = False,
 ) -> None:
@@ -1464,7 +1538,7 @@ async def _get_tracker_events_to_plot(
     training_trackers = await _get_training_trackers(file_importer, domain)
     number_of_trackers = len(training_trackers)
     if number_of_trackers > MAX_NUMBER_OF_TRAINING_STORIES_FOR_VISUALIZATION:
-        rasa.cli.utils.print_warning(
+        rasa.shared.utils.cli.print_warning(
             f"You have {number_of_trackers} different story paths in "
             f"your training data. Visualizing them is very resource "
             f"consuming. Hence, the visualization will only show the stories "
@@ -1497,10 +1571,11 @@ def _serve_application(
     file_importer: TrainingDataImporter,
     skip_visualization: bool,
     conversation_id: Text,
+    port: int,
 ) -> Sanic:
     """Start a core server and attach the interactive learning IO."""
 
-    endpoint = EndpointConfig(url=DEFAULT_SERVER_URL)
+    endpoint = EndpointConfig(url=DEFAULT_SERVER_FORMAT.format("http", port))
 
     async def run_interactive_io(running_app: Sanic) -> None:
         """Small wrapper to shut down the server once cmd io is done."""
@@ -1520,12 +1595,12 @@ def _serve_application(
 
     update_sanic_log_level()
 
-    app.run(host="0.0.0.0", port=DEFAULT_SERVER_PORT)
+    app.run(host="0.0.0.0", port=port)
 
     return app
 
 
-def start_visualization(image_path: Text = None) -> None:
+def start_visualization(image_path: Text, port: int) -> None:
     """Add routes to serve the conversation visualization files."""
 
     app = Sanic(__name__)
@@ -1551,14 +1626,16 @@ def start_visualization(image_path: Text = None) -> None:
 
     update_sanic_log_level()
 
-    app.run(host="0.0.0.0", port=DEFAULT_SERVER_PORT + 1, access_log=False)
+    app.run(host="0.0.0.0", port=port, access_log=False)
 
 
 # noinspection PyUnusedLocal
 async def train_agent_on_start(
     args, endpoints, additional_arguments, app, loop
 ) -> None:
-    _interpreter = NaturalLanguageInterpreter.create(endpoints.nlu or args.get("nlu"))
+    _interpreter = rasa.core.interpreter.create_interpreter(
+        endpoints.nlu or args.get("nlu")
+    )
 
     model_directory = args.get("out", tempfile.mkdtemp(suffix="_core_model"))
 
@@ -1620,16 +1697,22 @@ def run_interactive_learning(
     if server_args.get("domain"):
         PATHS["domain"] = server_args["domain"]
 
+    port = server_args.get("port", DEFAULT_SERVER_PORT)
+
     SAVE_IN_E2E = server_args["e2e"]
 
     if not skip_visualization:
-        p = Process(target=start_visualization, args=(DEFAULT_STORY_GRAPH_FILE,))
+        visualisation_port = port + 1
+        p = Process(
+            target=start_visualization,
+            args=(DEFAULT_STORY_GRAPH_FILE, visualisation_port),
+        )
         p.daemon = True
         p.start()
     else:
         p = None
 
-    app = run.configure_app(enable_api=True, conversation_id="default")
+    app = run.configure_app(port=port, conversation_id="default", enable_api=True)
     endpoints = AvailableEndpoints.read_endpoints(server_args.get("endpoints"))
 
     # before_server_start handlers make sure the agent is loaded before the
@@ -1639,7 +1722,9 @@ def run_interactive_learning(
         "before_server_start",
     )
 
-    _serve_application(app, file_importer, skip_visualization, conversation_id)
+    telemetry.track_interactive_learning_start(skip_visualization, SAVE_IN_E2E)
+
+    _serve_application(app, file_importer, skip_visualization, conversation_id, port)
 
     if not skip_visualization and p is not None:
         p.terminate()  # pytype: disable=attribute-error
