@@ -2,7 +2,8 @@ import logging
 import os
 from typing import Text, Dict
 import typing
-import rasa.utils.common as rasa_utils
+
+import rasa.shared.utils.io
 from rasa.constants import (
     ENV_GPU_CONFIG,
     ENV_CPU_INTER_OP_CONFIG,
@@ -35,7 +36,7 @@ def _setup_gpu_environment() -> None:
             _allocate_gpu_memory(physical_gpus[gpu_id], gpu_id_memory)
 
     else:
-        rasa_utils.raise_warning(
+        rasa.shared.utils.io.raise_warning(
             f"You have an environment variable '{ENV_GPU_CONFIG}' set but no GPUs were detected to configure."
         )
 
@@ -95,7 +96,7 @@ def _parse_gpu_config(gpu_memory_config: Text) -> Dict[int, int]:
         # Helper explanation of where the error comes from
         raise ValueError(
             f"Error parsing GPU configuration. Please cross-check the format of '{ENV_GPU_CONFIG}' "
-            f"at https://rasa.com/docs/rasa/api/tensorflow_usage.html#restricting-absolute-gpu-memory-available ."
+            f"at https://rasa.com/docs/rasa/tuning-your-model#restricting-absolute-gpu-memory-available ."
         )
 
     return parsed_gpu_config
